@@ -18,56 +18,95 @@ _LOGGER = logging.getLogger(__name__)
 class EnedisAnalytics:
     """Data analaytics."""
 
+    local_timezone = dt.now().astimezone().tzinfo
+
     def __init__(self, data: Collection[Collection[str]]) -> None:
         """Initialize Dataframe."""
         self.df = pd.DataFrame(data)
 
-    def get_data_analytcis(
+    def get_data_analytics(
         self,
         convertKwh: bool = False,
         convertUTC: bool = False,
         start_date: str | None = None,
         intervals: list[Tuple[str, str]] | None = None,
-        groupby: str | None = None,
+        groupby: bool = False,
         freq: str = "H",
         summary: bool = False,
         cumsum: float = 0,
+        prices: tuple[float, float] | None = None,
     ) -> Any:
         """Convert datas to analyze."""
         if not self.df.empty:
+            # Convert str to datetime
+            self.df.date = pd.to_datetime(self.df.date, format="%Y-%m-%d %H:%M:%S")
+            self.df.date = self.df.date.dt.tz_localize(self.local_timezone)
+
             if convertUTC:
-                self.df["date"] = pd.to_datetime(
-                    self.df["date"], utc=True, format="%Y-%m-%d %H:%M:%S"
-                )
-            else:
-                self.df["date"] = pd.to_datetime(
-                    self.df["date"], utc=False, format="%Y-%m-%d %H:%M:%S"
+                self.df.date = pd.to_datetime(
+                    self.df.date, utc=True, format="%Y-%m-%d %H:%M:%S"
                 )
 
-            self.df["date"] = self.df["date"].transform(self._midnightminus)
+            # Substract 1 minute at midnight
+            # because Pandas considers midnight as the next day while
+            # for Enedis it is the day before
+            if "interval_length" in self.df:
+                self.df.date = self.df.date.transform(self._midnightminus)
 
             if start_date:
-                self.df = self.df[(self.df["date"] > f"{start_date} 23:59:59")]
+                self.df = self.df[(self.df.date > f"{start_date} 23:59:59")]
 
-            self.df.index = self.df["date"]
+            self.df.index = self.df.date
+
+            # Add mark
+            self.df["notes"] = "HP"
 
         if self.df.empty:
             return self.df.to_dict(orient="records")
 
         if self.df.get("interval_length") is not None:
-            self.df["interval_length"] = self.df["interval_length"].transform(
+            self.df.interval_length = self.df.interval_length.transform(
                 self._weighted_interval
             )
         else:
-            self.df["interval_length"] = 1
+            self.df.interval_length = 1
 
         if convertKwh:
-            self.df["value"] = (
-                pd.to_numeric(self.df["value"]) / 1000 * self.df["interval_length"]
+            self.df.value = (
+                pd.to_numeric(self.df.value) / 1000 * self.df.interval_length
             )
+        else:
+            self.df.value = pd.to_numeric(self.df.value) * self.df.interval_length
 
         if intervals:
-            self.df = self._get_data_interval(intervals, groupby, freq, summary, cumsum)
+            self.df = self._get_data_interval(intervals)
+
+        if groupby:
+            self.df = (
+                self.df.groupby(["notes", pd.Grouper(key="date", freq=freq)])["value"]
+                .sum()
+                .reset_index()
+            )
+
+        if prices:
+            self.df.loc[(self.df.notes == "HP"), "price"] = self.df.value * prices[0]
+            self.df.loc[(self.df.notes == "HC"), "price"] = self.df.value * prices[1]
+
+        if summary:
+            self.df.loc[(self.df.notes == "HP"), "sum_value"] = (
+                self.df[(self.df.notes == "HP")].value.cumsum() + cumsum
+            )
+            self.df.loc[(self.df.notes == "HC"), "sum_value"] = (
+                self.df[(self.df.notes == "HC")].value.cumsum() + cumsum
+            )
+
+        if prices and summary:
+            self.df.loc[(self.df.notes == "HP"), "sum_price"] = (
+                self.df[(self.df.notes == "HP")].price.cumsum() + 100
+            )
+            self.df.loc[(self.df.notes == "HC"), "sum_price"] = (
+                self.df[(self.df.notes == "HC")].price.cumsum() + 100
+            )
 
         return self.df.to_dict(orient="records")
 
@@ -87,58 +126,29 @@ class EnedisAnalytics:
             return dt_date
         return dt_date
 
-    def _get_data_interval(
-        self,
-        intervalls: list[Tuple[str, str]],
-        groupby: str | None = None,
-        freq: str = "H",
-        summary: bool = False,
-        cumsum: float = 0,
-    ) -> pd.DataFrame:
+    def _get_data_interval(self, intervalls: list[Tuple[str, str]]) -> pd.DataFrame:
         """Group date from range time."""
-        in_df = pd.DataFrame()
         for intervall in intervalls:
-            start = pd.to_datetime(intervall[0]).time()
-            end = self._midnightminus(pd.to_datetime(intervall[1])).time()
-            df2 = self.df[
-                (self.df.date.dt.time > start) & (self.df.date.dt.time <= end)
-            ]
-            in_df = pd.concat([in_df, df2], ignore_index=True)
+            # Convert str to datetime
+            start = pd.to_datetime(intervall[0], format="%H:%M:%S")
+            end = pd.to_datetime(intervall[1], format="%H:%M:%S")
+            # Get Time
+            start = start.time()
+            end = self._midnightminus(end).time()
+            # Mark
+            self.df.loc[
+                (self.df.date.dt.time >= start) & (self.df.date.dt.time < end), "notes"
+            ] = "HC"
 
-        if groupby:
-            in_df = (
-                in_df.groupby(pd.Grouper(key="date", freq=freq))["value"]
-                .sum()
-                .reset_index()
-            )
-            in_df = in_df[in_df.value != 0]
-
-        if summary:
-            in_df["sum_value"] = in_df.value.cumsum() + cumsum
-
-        return in_df
-
-    def set_price(
-        self,
-        data: dict[str, Any],
-        price: float,
-        summary: bool = False,
-    ) -> Any:
-        """Set prince."""
-        df = pd.DataFrame(data)
-        if df.empty:
-            return df.to_dict(orient="records")
-        df["price"] = df["value"] * price
-        if summary:
-            df["sum_price"] = df["price"].cumsum()
-        return df.to_dict("records")
+        return self.df
 
     def get_last_value(self, data: dict[str, Any], orderby: str, value: str) -> Any:
         """Return last value after order by."""
         df = pd.DataFrame(data)
         if not df.empty:
             df = df.sort_values(by=orderby)
-            return df[value].iloc[-1]  # pylint: disable=unsubscriptable-object
+            if value in df.columns:
+                return df[value].iloc[-1]  # pylint: disable=unsubscriptable-object
 
 
 class EnedisByPDL:
@@ -152,10 +162,6 @@ class EnedisByPDL:
         self.offpeaks: list[str] = []
         self.dt_offpeak: list[dt] = []
         self.last_access: date | None = None
-        self.contract: dict[str, Any] = {}
-        self.tempo_day: str | None = None
-        self.ecowatt: dict[str, Any] = {}
-        self.valid_access: dict[str, Any] = {}
 
     async def async_fetch_datas(
         self, service: str, pdl: str, start: dt | None = None, end: dt | None = None
@@ -180,6 +186,11 @@ class EnedisByPDL:
         """Return valid access."""
         return await self.async_fetch_datas("valid_access", pdl)
 
+    async def async_has_access(self, pdl: str) -> bool:
+        """Check valid access."""
+        access = await self.async_valid_access(pdl)
+        return access.get("valid", False) is True
+
     async def async_get_contract(self, pdl: str) -> Any:
         """Return contract information."""
         contract = {}
@@ -217,15 +228,33 @@ class EnedisByPDL:
         """Return all adresses information."""
         return await self.async_fetch_datas("adresses", pdl)
 
-    async def async_get_tempoday(self) -> Any:
+    async def async_get_tempoday(
+        self, start: dt | None = None, end: dt | None = None
+    ) -> Any:
         """Return Tempo Day."""
-        day = dt.now().strftime("%Y-%m-%d")
-        return await self.auth.request(path=f"rte/tempo/{day}/{day}")
+        str_start = (
+            start.strftime("%Y-%m-%d") if start else dt.now().strftime("%Y-%m-%d")
+        )
+        str_end = (
+            end.strftime("%Y-%m-%d")
+            if end
+            else (dt.now() + timedelta(days=1)).strftime("%Y-%m-%d")
+        )
+        return await self.auth.request(path=f"rte/tempo/{str_start}/{str_end}")
 
-    async def async_get_ecowatt(self) -> Any:
+    async def async_get_ecowatt(
+        self, start: dt | None = None, end: dt | None = None
+    ) -> Any:
         """Return Ecowatt information."""
-        day = dt.now().strftime("%Y-%m-%d")
-        return await self.auth.request(path=f"rte/ecowatt/{day}/{day}")
+        str_start = (
+            start.strftime("%Y-%m-%d") if start else dt.now().strftime("%Y-%m-%d")
+        )
+        str_end = (
+            end.strftime("%Y-%m-%d")
+            if end
+            else (dt.now() + timedelta(days=1)).strftime("%Y-%m-%d")
+        )
+        return await self.auth.request(path=f"rte/ecowatt/{str_start}/{str_end}")
 
     async def async_has_offpeak(self, pdl: str) -> bool:
         """Has offpeak hours."""
